"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Jwt = void 0;
const utils_1 = require("@sd-jwt/utils");
const error_1 = require("./error");
const utils_2 = require("@sd-jwt/utils");
const decode_1 = require("@sd-jwt/decode");
class Jwt {
    constructor(options, additionalOptions) {
        this.header = options === null || options === void 0 ? void 0 : options.header;
        this.payload = options === null || options === void 0 ? void 0 : options.payload;
        this.signature = options === null || options === void 0 ? void 0 : options.signature;
        this.signer = additionalOptions === null || additionalOptions === void 0 ? void 0 : additionalOptions.signer;
    }
    /**
     *
     * Instantiate a JWT from a compact format.
     *
     * Two generics may be supplied for typing on the Header and Payload. These are not enforced.
     *
     * Defined in: {@link https://datatracker.ietf.org/doc/html/rfc7519#section-3 | RFC 7519 }
     *
     */
    static fromCompact(compact) {
        const { header, payload, signature } = (0, decode_1.jwtFromCompact)(compact);
        const jwt = new Jwt({
            header,
            payload,
            signature
        });
        jwt.compact = compact;
        return jwt;
    }
    /**
     *
     * Replaces the current Header a new one.
     *
     */
    withHeader(header) {
        this.header = header;
        return this;
    }
    /**
     *
     * Add a new claim to the Header, overriding the old one if it already is on there.
     *
     */
    addHeaderClaim(item, value) {
        var _a;
        (_a = this.header) !== null && _a !== void 0 ? _a : (this.header = {});
        if (value !== undefined && item !== null) {
            this.header = Object.assign(Object.assign({}, this.header), { [item]: value });
        }
        return this;
    }
    /**
     *
     * Replaces the current Payload a new one.
     *
     */
    withPayload(payload) {
        this.payload = payload;
        return this;
    }
    /**
     *
     * Add a new claim to the Payload, overriding the old one if it already is on there.
     *
     */
    addPayloadClaim(item, value) {
        var _a;
        (_a = this.payload) !== null && _a !== void 0 ? _a : (this.payload = {});
        if (value !== undefined && item !== null) {
            this.payload = Object.assign(Object.assign({}, this.payload), { [item]: value });
        }
        return this;
    }
    /**
     *
     * Manually append a signature to the JWT. This signature is not validated.
     *
     * @note Only use this if the supplying a signing callback does not fit your use case.
     *
     */
    withSignature(signature) {
        this.signature = signature;
        return this;
    }
    /**
     *
     * Add a signing callback to the JWT that will be used for creating the signature.
     *
     */
    withSigner(signer) {
        this.signer = signer;
        return this;
    }
    /**
     *
     * Assert that there is a Header on the JWT.
     *
     * @throws when the Header is not defined
     *
     */
    assertHeader() {
        if (this.header)
            return;
        throw new error_1.JwtError('Header must be defined');
    }
    /**
     *
     * Assert that there is a Payload on the JWT.
     *
     * @throws when the Payload is not defined
     *
     */
    assertPayload() {
        if (this.payload)
            return;
        throw new error_1.JwtError('Payload must be defined');
    }
    /**
     *
     * Assert that there is a Signature on the JWT.
     *
     * @throws when the Signature is not defined
     *
     */
    assertSignature() {
        if (this.signature)
            return;
        throw new error_1.JwtError('Signature must be defined');
    }
    /**
     *
     * Assert that there is a Signing callback on the JWT.
     *
     * @throws when the Signer is not defined
     *
     */
    assertSigner() {
        if (this.signer)
            return;
        throw new error_1.JwtError('A signer must be provided to create a signature. You can set it with this.withSigner()');
    }
    /**
     *
     * Assert that there is a specific claim, possibly with value, in the Header.
     *
     */
    assertClaimInHeader(claimKey, claimValue) {
        this.assertHeader();
        try {
            this.assertClaimInObject(this.header, claimKey, claimValue);
        }
        catch (e) {
            if (e instanceof error_1.JwtError) {
                e.message += ' within the header';
            }
            throw e;
        }
    }
    /**
     *
     * Assert that there is a specific claim, possibly with value, in the Payload.
     *
     */
    assertClaimInPayload(claimKey, claimValue) {
        this.assertPayload();
        try {
            this.assertClaimInObject(this.payload, claimKey, claimValue);
        }
        catch (e) {
            if (e instanceof error_1.JwtError) {
                e.message += ' within the payload';
            }
            throw e;
        }
    }
    assertClaimInObject(object, claimKey, claimValue) {
        const value = (0, utils_2.getValueByKeyAnyLevel)(object, claimKey);
        if (!value) {
            throw new error_1.JwtError(`Claim key '${claimKey}' not found in any level`);
        }
        if (claimValue && !(0, utils_2.simpleDeepEqual)(value, claimValue)) {
            throw new error_1.JwtError(`Claim key '${claimKey}' was found, but values did not match`);
        }
    }
    /**
     *
     * Get a claim within the payload.
     *
     * @throws when the payload is not defined
     * @throws when the claim could not be found at any level
     *
     */
    getClaimInPayload(claimKey) {
        this.assertPayload();
        return this.getClaimInObject(this.payload, claimKey);
    }
    /**
     *
     * Get a claim within the payload.
     *
     * @throws when the payload is not defined
     * @throws when the claim could not be found at any level
     *
     */
    getClaimInHeader(claimKey) {
        this.assertHeader();
        return this.getClaimInObject(this.header, claimKey);
    }
    getClaimInObject(object, claimKey) {
        const value = (0, utils_2.getValueByKeyAnyLevel)(object, claimKey);
        if (!value) {
            throw new error_1.JwtError(`Claim key '${claimKey}' not found in any level`);
        }
        return value;
    }
    /**
     *
     * Returns a string of what needs to be signed.
     *
     * Defined in: {@link https://datatracker.ietf.org/doc/html/rfc7519#section-3 | RFC 7519}
     *
     */
    get signableInput() {
        this.assertHeader();
        this.assertPayload();
        return `${this.compactHeader}.${this.compactPayload}`;
    }
    /**
     *
     * Sign the Header and Payload and append the signature to the JWT.
     *
     */
    signAndAdd() {
        return __awaiter(this, void 0, void 0, function* () {
            this.assertSigner();
            const signature = yield this.signer(this.signableInput, this.header);
            this.withSignature(signature);
            return this;
        });
    }
    get compactHeader() {
        this.assertHeader();
        return utils_1.Base64url.encodeFromJson(this.header);
    }
    get compactPayload() {
        this.assertPayload();
        return utils_1.Base64url.encodeFromJson(this.payload);
    }
    /**
     *
     * Create a compact format of the JWT.
     *
     * This will add a signature if there is none.
     *
     * @throws When the signature and signer are not defined
     *
     */
    toCompact() {
        return __awaiter(this, void 0, void 0, function* () {
            this.assertHeader();
            this.assertPayload();
            if (!this.signature) {
                yield this.signAndAdd();
            }
            const encodedSignature = utils_1.Base64url.encode(this.signature);
            return `${this.compactHeader}.${this.compactPayload}.${encodedSignature}`;
        });
    }
    /**
     *
     * Verify the JWT.
     *
     * - Check the nbf claim with `now`
     * - Check the exp claim with `now`
     * - Additionally validate any required claims
     * - Additionally pass in a specific publicKeyJwk to validate the signature
     *
     */
    verify(verifySignature, requiredClaims, publicKeyJwk) {
        return __awaiter(this, void 0, void 0, function* () {
            this.assertHeader();
            this.assertPayload();
            this.assertSignature();
            const ret = {};
            ret.isSignatureValid = yield verifySignature({
                header: this.header,
                signature: this.signature,
                message: this.signableInput,
                publicKeyJwk
            });
            if ('nbf' in this.payload) {
                const now = new Date();
                const notBefore = new Date(this.payload.nbf * 1000);
                ret.isNotBeforeValid = notBefore < now;
            }
            if ('exp' in this.payload) {
                const now = new Date();
                const expiryTime = new Date(this.payload.exp * 1000);
                ret.isExpiryTimeValid = expiryTime > now;
            }
            if (requiredClaims) {
                ret.areRequiredClaimsIncluded = requiredClaims.every((claim) => claim in this.payload);
            }
            ret.isValid = Object.values(ret)
                .filter((i) => typeof i === 'boolean')
                .every((i) => !!i);
            return ret;
        });
    }
}
exports.Jwt = Jwt;
//# sourceMappingURL=jwt.js.map