"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getDisclosuresForPresentationFrame = void 0;
const utils_1 = require("../utils");
const error_1 = require("./error");
const disclosureMapping_1 = require("./disclosureMapping");
const getDisclosuresForPresentationFrame = (signedPayload, presentationFrame, prettyClaims, disclosures = []) => {
    const requiredDisclosureDigests = new Set();
    const disclosureMap = (0, disclosureMapping_1.getDisclosureMap)(disclosures);
    const payloadDisclosureMapping = (0, disclosureMapping_1.getPayloadDisclosureMapping)(signedPayload, disclosureMap);
    // No disclosures needed
    if (payloadDisclosureMapping === null) {
        if (disclosures.length > 0) {
            throw new error_1.SdJwtError('Payload disclosure mapping is null, but disclosures are present.');
        }
        return [];
    }
    for (const node of (0, utils_1.traverseNodes)(presentationFrame)) {
        // We only want to process leaf nodes here
        if (!node.isLeaf)
            continue;
        if (typeof node.value !== 'boolean') {
            throw new error_1.SdJwtError(`Expected leaf value in presentation frame to be of type boolean, but found ${typeof node.value}`);
        }
        // If the value is false, it means we don't want to disclose it
        if (node.value === false)
            continue;
        if (!(0, utils_1.hasByPath)(prettyClaims, node.path)) {
            throw new error_1.SdJwtError(`Path ${node.path.join('.')} from presentation frame is not present in pretty SD-JWT payload. The presentation frame may only include properties that are present in the SD-JWT payload.`);
        }
        let path = [...node.path];
        while (!(0, utils_1.hasByPath)(payloadDisclosureMapping, path)) {
            if (path.pop() === undefined)
                break;
        }
        // There are no disclosures on this path, meaning the property is disclosed by default in the signed payload
        if (path.length === 0)
            continue;
        const disclosure = (0, utils_1.getByPath)(payloadDisclosureMapping, path);
        // If disclosure is string, it means it's already the digest
        if (typeof disclosure === 'string')
            requiredDisclosureDigests.add(disclosure);
        // Otherwise we want to get all the child digests as well
        else {
            for (const nestedItem of (0, utils_1.traverseNodes)(disclosure)) {
                if (!nestedItem.isLeaf ||
                    typeof nestedItem.value !== 'string') {
                    continue;
                }
                requiredDisclosureDigests.add(nestedItem.value);
            }
        }
    }
    for (const disclosureDigest of requiredDisclosureDigests.values()) {
        const disclosure = disclosureMap[disclosureDigest];
        if (!disclosure) {
            throw new Error('disclosure not found');
        }
        disclosure.parentDisclosures.forEach((d) => requiredDisclosureDigests.add(d.digest));
    }
    return Array.from(requiredDisclosureDigests).map((digest) => disclosureMap[digest].disclosure);
};
exports.getDisclosuresForPresentationFrame = getDisclosuresForPresentationFrame;
//# sourceMappingURL=presentationFrame.js.map